import { createTheme } from '../src/theme';
import { color, spacing, boxShadow, fromProp, styledBy, typography } from '../src/styles';

const theme = createTheme();
const prop = { theme, tone: '20', variant: 'success', isSelected: true };

describe('Given a Theme', () => {
  describe('When pick a color given static values', () => {
    it('Then it provides the correct color', () => {
      const pickGray50 = color('gray', '50');
      expect(pickGray50(prop)).toBe('#D9D9D9');
    });
  });
  describe('When pick a color based on dynamic value from props', () => {
    it('Then it provides the correct color', () => {
      const pickRed = color('red', fromProp('tone'));
      expect(pickRed(prop)).toBe('#FFF3F3');
    });
  });
  describe('When choosing styles based on a prop value', () => {
    it('Then it provides the correct color', () => {
      const pickBasedOnState = styledBy('variant', {
        'success': color('blue', fromProp('tone')),
        'error': color('red', fromProp('tone'))
      });
      expect(pickBasedOnState(prop)).toBe('#DCEDFF');
    });
  });
  describe('When choosing styles based on a computed prop value', () => {
    it('Then it provides the correct color', () => {
      function rowBackground(props: any) {
        if (props.isSelected) {
          return 'selected'
        }
        if (props.isRightClicked) {
          return 'rightClicked'
        }
        return false;
      }

      const pickBasedOnComputedProp = styledBy(rowBackground, {
        'selected': color('blue', fromProp('tone')),
        'rightClicked': color('red', fromProp('tone')),
        false: 'transparent',
      });

      expect(pickBasedOnComputedProp(prop)).toBe('#DCEDFF');
    });
  });
  describe('When pick a spacing', () => {
    it('Then it provides the correct space', () => {
      const pickSpace = spacing('xl');
      expect(pickSpace(prop)).toBe('24px');
    });
  });
  describe('When pick a box shadow', () => {
    it('Then it provides the correct box shadow', () => {
      const pickShadow = boxShadow(50);
      expect(pickShadow(prop)).toBe('box-shadow: 0px 10px 20px 0px rgba(0,0,0,19),0px 6px 6px 0px rgba(0,0,0,26);');
    });
  });
  describe('When pick a typography', () => {
    it('Then it provides the correct typography', () => {
      const pickTypography = typography('lg', 'medium', 'primary');
      const output = pickTypography(prop);
      expect(output.includes('font-size: 22px;')).toBeTruthy();
      expect(output.includes('line-height: 32px;')).toBeTruthy();
      expect(output.includes('letter-spacing: 0.15;')).toBeTruthy();
      expect(output.includes('font-weight: 500;')).toBeTruthy();
      expect(output.includes('color: #282828;')).toBeTruthy();
    });
  });
});
